package com.kmalbz.api.route

import com.kmalbz.database.service.ISampleObjectService
import io.ktor.application.call
import io.ktor.http.HttpStatusCode
import io.ktor.response.respond
import io.ktor.routing.Routing
import io.ktor.routing.get
import org.koin.ktor.ext.inject
import java.time.LocalDate
import java.time.format.DateTimeFormatter

/**
 * Sample Service
 * 
 * This is the sample interface of the Birbnetes system.
 */
class OutputServiceRDBServer {
    /**
     * sample
     */
    fun Routing.registerOutput() {
        val resultObjectService by inject<ISampleObjectService>()
        get("/sample"){
            call.respond(resultObjectService.getAllSampleObjects())
        }

        get("/sample/after/{dateAfter}") {
            val dateAfter = call.parameters["dateAfter"] ?: error(HttpStatusCode.NotAcceptable)
            val dateTimeFormatter: DateTimeFormatter = DateTimeFormatter.ISO_LOCAL_DATE
            val localDateAfter : LocalDate = LocalDate.parse(dateAfter,dateTimeFormatter)
            val resultList = resultObjectService.getSampleObjectafterDate(localDateAfter)

            call.respond(resultList)
        }

        get("/sample/before/{dateBefore}") {
            val dateAfter = call.parameters["dateBefore"] ?: error(HttpStatusCode.NotAcceptable)
            val dateTimeFormatter: DateTimeFormatter = DateTimeFormatter.ISO_LOCAL_DATE
            val localDateBefore : LocalDate = LocalDate.parse(dateAfter,dateTimeFormatter)
            val resultList = resultObjectService.getSampleObjectbeforeDate(localDateBefore)

            call.respond(resultList)
        }

        get("/sample/{tagID}") {
            val tagID = call.parameters["tagID"] ?: error(HttpStatusCode.NotAcceptable)
            val resultObject = resultObjectService.getSampleObjectbyTag(tagID) ?: call.respond(HttpStatusCode.NotFound)

            call.respond(resultObject)
        }
    }
}
